@@ -41,6 +41,11 @@ if is_service_enabled tacker; then
             tacker_check_and_download_images
             echo_summary "Registering default VIM"
             tacker_register_default_vim
+
+            if is_service_enabled ceilometer; then
+                echo_summary "Configure maintenance event types"
+                configure_maintenance_event_types
+            fi
         fi
     fi
 
