@@ -139,6 +139,18 @@ class Invalid(TackerException):
     message = _("Bad Request - Invalid Parameters")
 
 
+class CreateApiFalse(TackerException):
+    message = _('Failed to create resource.')
+
+
+class InitApiFalse(TackerException):
+    message = _('Failed to init resource.')
+
+
+class ReadEndpoindsFalse(TackerException):
+    message = _('The method to read a resource failed.')
+
+
 class InvalidInput(BadRequest):
     message = _("Invalid input for operation: %(error_message)s.")
 
@@ -220,6 +232,11 @@ class VnfInstanceConflictState(Conflict):
                 "%(action)s while the vnf instance is in this state.")
 
 
+class VnfConflictState(Conflict):
+    message = _("Vnf %(uuid)s in %(attr)s %(state)s. Cannot "
+                "%(action)s while the vnf is in this state.")
+
+
 class FlavourNotFound(NotFound):
     message = _("No flavour with id '%(flavour_id)s'.")
 
@@ -248,6 +265,10 @@ class VnfSoftwareImageNotFound(NotFound):
     message = _("No vnf software image  with id %(id)s.")
 
 
+class VnfArtifactNotFound(NotFound):
+    message = _("No vnf artifact with id %(id)s.")
+
+
 class VnfInstantiatedInfoNotFound(NotFound):
     message = _("No vnf instantiated info for vnf id %(vnf_instance_id)s.")
 
@@ -287,7 +308,7 @@ class VNFPackageURLInvalid(Invalid):
 
 
 class VnfPackageLocationInvalid(Invalid):
-    message = _("Failed to find location: %(location)")
+    message = _("Failed to find location: %(location)s")
 
 
 class InvalidZipFile(Invalid):
@@ -303,6 +324,10 @@ class FailedToGetVnfdData(Invalid):
     message = _("Failed to get csar zip file from glance store: %(error)s")
 
 
+class FailedToGetVnfArtifact(Invalid):
+    message = _("Failed to get artifact file from glance store: %(error)s")
+
+
 class FailedToGetVnfPackageDetails(Invalid):
     message = _("Failed to get vnf package details: %(error)s")
 
@@ -322,6 +347,18 @@ class LimitExceeded(TackerException):
         super(LimitExceeded, self).__init__(*args, **kwargs)
 
 
+class NotificationProcessingError(TackerException):
+    message = _("Notification Processing Failed: %(error)s")
+
+
 class UserDataUpdateCreateFailed(TackerException):
     msg_fmt = _("User data for VNF package  %(id)s cannot be updated "
                 "or created after %(retries)d retries.")
+
+
+class DBAccessError(TackerException):
+    message = _("DB Access Error")
+
+
+class SeeOther(TackerException):
+    code = 303
