@@ -17,7 +17,6 @@ from oslo_db.exception import DBDuplicateEntry
 from oslo_log import log as logging
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
-from six import iteritems
 
 import sqlalchemy as sa
 from sqlalchemy import orm
@@ -301,7 +300,7 @@ class NSPluginDb(network_service.NSPluginBase, db_base.CommonDbMixin):
         vnf_ids = dict()
         vnffg_ids = dict()
         if len(output) > 0:
-            for vnfd_name, vnfd_val in iteritems(vnfd_dict):
+            for vnfd_name, vnfd_val in vnfd_dict.items():
                 for instance in vnfd_val['instances']:
                     if 'mgmt_ip_address_' + instance in output:
                         mgmt_ip_addresses[instance] = ast.literal_eval(
