@@ -19,6 +19,7 @@ import yaml
 
 from collections import OrderedDict
 from oslo_log import log as logging
+from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
 
 from tacker._i18n import _
@@ -93,11 +94,13 @@ deletenodes = (MONITORING, FAILURE, PLACEMENT)
 
 HEAT_RESOURCE_MAP = {
     "flavor": "OS::Nova::Flavor",
-    "image": "OS::Glance::WebImage"
+    "image": "OS::Glance::WebImage",
+    "maintenance": "OS::Aodh::EventAlarm"
 }
 
 SCALE_GROUP_RESOURCE = "OS::Heat::AutoScalingGroup"
 SCALE_POLICY_RESOURCE = "OS::Heat::ScalingPolicy"
+PLACEMENT_POLICY_RESOURCE = "OS::Nova::ServerGroup"
 
 
 @log.log
@@ -182,7 +185,7 @@ def get_vdu_applicationmonitoring(template):
     for node in node_list:
         nt = template[tpl_temp][n_temp][node]
         if nt['type'] == TACKERVDU:
-            if poly in nt['properties'].keys():
+            if poly in nt['properties']:
                 mon_policy = nt['properties'][poly]
                 if mon_policy != 'noop':
                     policy_dict['vdus'][node] = {}
@@ -258,6 +261,12 @@ def pre_process_alarm_resources(vnf, template, vdu_metadata, unique_id=None):
                 'before_end_actions': {
                     'event_type': 'lease.event.before_end_lease'},
                 'end_actions': {'event_type': 'lease.event.end_lease'}}
+    maintenance_actions = _process_alarm_actions_for_maintenance(vnf)
+    if maintenance_actions:
+        alarm_actions.update(maintenance_actions)
+        alarm_resources['event_types'] = {}
+        alarm_resources['event_types'].update({
+            'ALL_maintenance': {'event_type': 'maintenance.scheduled'}})
     alarm_resources['query_metadata'] = query_metadata
     alarm_resources['alarm_actions'] = alarm_actions
     return alarm_resources
@@ -337,6 +346,22 @@ def _process_alarm_actions_for_reservation(vnf, policy):
     return alarm_actions
 
 
+def _process_alarm_actions_for_maintenance(vnf):
+    # process alarm url here
+    alarm_actions = dict()
+    maintenance_props = vnf['attributes'].get('maintenance', '{}')
+    maintenance_props = jsonutils.loads(maintenance_props)
+    maintenance_url = vnf['attributes'].get('maintenance_url', '')
+    for vdu, access_key in maintenance_props.items():
+        action = '%s_maintenance' % vdu
+        alarm_url = '%s/%s' % (maintenance_url.rstrip('/'), access_key)
+        if alarm_url:
+            LOG.debug('Alarm url in heat %s', alarm_url)
+            alarm_actions[action] = dict()
+            alarm_actions[action]['alarm_actions'] = [alarm_url]
+    return alarm_actions
+
+
 def get_volumes(template):
     volume_dict = dict()
     node_tpl = template['topology_template']['node_templates']
@@ -423,6 +448,8 @@ def add_resources_tpl(heat_dict, hot_res_tpl):
                 "properties": {}
             }
 
+            if res == "maintenance":
+                continue
             for prop, val in (vdu_dict).items():
                 # change from 'get_input' to 'get_param' to meet HOT template
                 if isinstance(val, dict):
@@ -517,6 +544,7 @@ def post_process_heat_template(heat_tpl, mgmt_ports, metadata,
             if heat_dict['resources'].get(vdu_name):
                 heat_dict['resources'][vdu_name]['properties']['metadata'] =\
                     metadata_dict
+    add_resources_tpl(heat_dict, res_tpl)
 
     query_metadata = alarm_resources.get('query_metadata')
     alarm_actions = alarm_resources.get('alarm_actions')
@@ -532,16 +560,14 @@ def post_process_heat_template(heat_tpl, mgmt_ports, metadata,
     if alarm_actions:
         for trigger_name, alarm_actions_dict in alarm_actions.items():
             if heat_dict['resources'].get(trigger_name):
-                heat_dict['resources'][trigger_name]['properties']. \
-                    update(alarm_actions_dict)
-
+                heat_dict['resources'][trigger_name]['properties'].update(
+                    alarm_actions_dict)
     if event_types:
         for trigger_name, event_type in event_types.items():
             if heat_dict['resources'].get(trigger_name):
                 heat_dict['resources'][trigger_name]['properties'].update(
                     event_type)
 
-    add_resources_tpl(heat_dict, res_tpl)
     for res in heat_dict["resources"].values():
         if not res['type'] == HEAT_SOFTWARE_CONFIG:
             continue
@@ -951,7 +977,7 @@ def post_process_template(template):
             template.nodetemplates.remove(nt)
             continue
 
-        if nt.type in delpropmap.keys():
+        if nt.type in delpropmap:
             for prop in delpropmap[nt.type]:
                 for p in nt.get_properties_objects():
                     if prop == p.name:
@@ -959,16 +985,16 @@ def post_process_template(template):
 
         # change the property value first before the property key
         if nt.type in convert_prop_values:
-            for prop in convert_prop_values[nt.type].keys():
+            for prop in convert_prop_values[nt.type]:
                 for p in nt.get_properties_objects():
                     if (prop == p.name and
                             p.value in
-                            convert_prop_values[nt.type][prop].keys()):
+                            convert_prop_values[nt.type][prop]):
                         v = convert_prop_values[nt.type][prop][p.value]
                         p.value = v
 
         if nt.type in convert_prop:
-            for prop in convert_prop[nt.type].keys():
+            for prop in convert_prop[nt.type]:
                 for p in nt.get_properties_objects():
                     if prop == p.name:
                         schema_dict = {'type': p.type}
@@ -1038,6 +1064,15 @@ def findvdus(template):
     return vdus
 
 
+def find_maintenance_vdus(template):
+    maintenance_vdu_names = list()
+    vdus = findvdus(template)
+    for nt in vdus:
+        if nt.get_properties().get('maintenance'):
+            maintenance_vdu_names.append(nt.name)
+    return maintenance_vdu_names
+
+
 def get_flavor_dict(template, flavor_extra_input=None):
     flavor_dict = {}
     vdus = findvdus(template)
@@ -1128,9 +1163,9 @@ def get_image_dict(template):
             continue
         artifacts = vdu.entity_tpl["artifacts"]
         for name, artifact in (artifacts).items():
-            if ('type' in artifact.keys() and
+            if ('type' in artifact and
                artifact["type"] == IMAGE):
-                if 'file' not in artifact.keys():
+                if 'file' not in artifact:
                     raise vnfm.FilePathMissing()
                 image_dict[vdu.name] = {
                     "location": artifact["file"],
@@ -1152,6 +1187,27 @@ def get_resources_dict(template, flavor_extra_input=None):
     return res_dict
 
 
+def add_maintenance_resources(template, res_tpl):
+    res_dict = {}
+    maintenance_vdus = find_maintenance_vdus(template)
+    maintenance_vdus.append('ALL')
+    if maintenance_vdus:
+        for vdu_name in maintenance_vdus:
+            res_dict[vdu_name] = {}
+        res_tpl['maintenance'] = res_dict
+
+
+@log.log
+def get_policy_dict(template, policy_type):
+    policy_dict = dict()
+    for policy in template.policies:
+        if (policy.type_definition.is_derived_from(policy_type)):
+            policy_attrs = dict()
+            policy_attrs['targets'] = policy.targets
+            policy_dict[policy.name] = policy_attrs
+    return policy_dict
+
+
 @log.log
 def get_scaling_policy(template):
     scaling_policy_names = list()
