@@ -16,7 +16,6 @@
 """
 Utility methods for working with WSGI servers
 """
-from __future__ import print_function
 import functools
 
 import errno
@@ -29,6 +28,8 @@ import time
 import eventlet.wsgi
 # eventlet.patcher.monkey_patch(all=False, socket=True, thread=True)
 from oslo_config import cfg
+import tacker.conf
+
 import oslo_i18n as i18n
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
@@ -76,13 +77,14 @@ socket_opts = [
                       "the server securely")),
 ]
 
-CONF = cfg.CONF
+CONF = tacker.conf.CONF
 CONF.register_opts(socket_opts)
 
 
 def config_opts():
     return [(None, socket_opts)]
 
+
 LOG = logging.getLogger(__name__)
 
 
@@ -130,6 +132,12 @@ def expected_errors(errors):
                     # Handle an authorized exception, will be
                     # automatically converted to a HTTP 401.
                     raise
+                elif isinstance(exc, exception.Conflict):
+                    # Note(tpatil): Handle a conflict error, which
+                    # happens due to resources in wrong state.
+                    # ResourceExceptionHandler silently converts Conflict
+                    # to HTTPConflict
+                    raise
 
                 LOG.exception("Unexpected exception in API method")
                 msg = _('Unexpected API Error. Please report this at '
@@ -396,7 +404,7 @@ class Request(webob.Request):
         type_from_header = self.get_content_type()
         if type_from_header:
             return type_from_header
-        ctypes = ['application/json']
+        ctypes = ['application/json', 'text/plain', 'application/zip']
 
         # Finally search in Accept-* headers
         bm = self.accept.best_match(ctypes)
@@ -861,6 +869,9 @@ class ResourceExceptionHandler(object):
             raise Fault(exception.ConvertedException(
                 code=ex_value.code,
                 explanation=ex_value.format_message()))
+        elif isinstance(ex_value, exception.Conflict):
+            raise Fault(webob.exc.HTTPConflict(
+                explanation=ex_value.format_message()))
         elif isinstance(ex_value, TypeError):
             exc_info = (ex_type, ex_value, ex_traceback)
             LOG.error('Exception handling resource: %s', ex_value,
@@ -935,9 +946,11 @@ class ResponseObject(object):
         """
 
         serializer = self.serializer
-
-        body = None
-        if self.obj is not None:
+        if self.obj is None:
+            body = None
+        elif content_type == 'text/plain':
+            body = self.obj
+        else:
             body = serializer.serialize(self.obj)
         response = webob.Response(body=body)
         response.status_int = self.code
@@ -1027,12 +1040,13 @@ class Resource(Application):
                 action_result = self.dispatch(request, action, args)
         except Fault as ex:
             response = ex
-        except Exception as ex:
+        except Exception:
             raise Fault(webob.exc.HTTPInternalServerError())
 
         if not response:
             resp_obj = None
-            if type(action_result) is dict or action_result is None:
+            if (isinstance(action_result, (dict, list, str)) or
+                    action_result is None):
                 resp_obj = ResponseObject(action_result)
             elif isinstance(action_result, ResponseObject):
                 resp_obj = action_result
